package cs636.music.domain;

/**
 * CartItem: memory-only object for item in the cart
 * i.e, it is not a "persistent object" tied to database data
 * It is mutable, but also user private, so
 * it doesn't violate the principle of keeping
 * persistent domain objects out of the presentation layer.
 * Note that the product object, a persistent object is
 * only represented by its code, not the whole object.
 * Data is moved to a LineItem object in checkout
 *
 */
public class CartItem {

	private String productCode;
	private int quantity;
	
	// no-args constructor, to be proper JavaBean
	public CartItem() {}
	
	public CartItem (String productCode, int quantity) {
		this.productCode = productCode;
		this.quantity = quantity;
	}
	
	public String getProductCode() {
		return productCode;
	}
	
	public int getQuantity() {
		return quantity;
	}
	
	public void setQuantity(int quantity) {
		this.quantity = quantity;
	}

}
